@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21373 $
- * $Author: ms30002000 $
- * $Date: 2008-10-16 19:00:01 +0000 (Thu, 16 Oct 2008) $
+ * $Revision: 21804 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-16 02:25:58 +0000 (Tue, 16 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -39,6 +39,7 @@
 #include <ctype.h>
 #include <ptlib/pfactory.h>
 #include <ptlib/pprocess.h>
+#include <ptlib/svcproc.h>
 
 #ifdef _WIN32
 #include <ptlib/msos/ptlib/debstrm.h>
@@ -72,6 +73,27 @@ namespace PWLibStupidLinkerHacks {
 };
 #endif
 
+class PExternalThread : public PThread
+{
+  PCLASSINFO(PExternalThread, PThread);
+  public:
+    PExternalThread()
+    {
+      SetThreadName(PString::Empty());
+      PTRACE(5, "PTLib\tCreated external thread " << this << " for id " << GetCurrentThreadId());
+    }
+
+    ~PExternalThread()
+    {
+      PTRACE(5, "PTLib\tDestroyed external thread " << this << " for id " << GetThreadId());
+    }
+
+    void Main()
+    {
+    }
+};
+
+
 class PSimpleThread : public PThread
 {
     PCLASSINFO(PSimpleThread, PThread);
@@ -532,10 +554,11 @@ ostream & PTrace::End(ostream & paramStream)
   }
   else {
 #ifdef _WIN32_WCE
-    *info.stream << "\r\n";
-#else
-    *info.stream << '\n';
+    if (!PIsDescendant(info.stream, PDebugStream) && info.stream != &cerr)
+      *info.stream << "\r\n";
+    else
 #endif
+    *info.stream << '\n';
   }
   info.stream->flush();
 
@@ -1455,24 +1478,33 @@ PString PConfigArgs::CharToString(char ch) const
 // PProcess
 
 PProcess * PProcessInstance;
-int PProcess::p_argc;
-char ** PProcess::p_argv;
-char ** PProcess::p_envp;
 
 typedef std::map<PString, PProcessStartup *> PProcessStartupList;
 
-int PProcess::_main(void *)
+int PProcess::InternalMain(void *)
 {
   Main();
   return terminationValue;
 }
 
 
-void PProcess::PreInitialise(int c, char ** v, char ** e)
+void PProcess::PreInitialise(int c, char ** v, char **)
 {
-  p_argc = c;
-  p_argv = v;
-  p_envp = e;
+  if (executableFile.IsEmpty()) {
+    PString execFile = v[0];
+    if (PFile::Exists(execFile))
+      executableFile = execFile;
+    else {
+      execFile += ".exe";
+      if (PFile::Exists(execFile))
+        executableFile = execFile;
+    }
+  }
+
+  if (productName.IsEmpty())
+    productName = executableFile.GetTitle().ToLower();
+
+  arguments.SetArgs(c-1, v+1);
 }
 
 
@@ -1485,15 +1517,20 @@ static PProcessStartupList & GetPProcessStartupList()
 
 PProcess::PProcess(const char * manuf, const char * name,
                            WORD major, WORD minor, CodeStatus stat, WORD build)
-  : manufacturer(manuf), productName(name)
+  : terminationValue(0)
+  , manufacturer(manuf)
+  , productName(name)
+  , majorVersion(major)
+  , minorVersion(minor)
+  , status(stat)
+  , buildNumber(build)
+  , maxHandles(INT_MAX)
+  , m_library(false)
 {
-  PProcessInstance = this;
-  terminationValue = 0;
+  activeThreads.DisallowDeleteObjects();
+  activeThreads.SetAt((PINDEX)GetCurrentThreadId(), this);
 
-  majorVersion = major;
-  minorVersion = minor;
-  status = stat;
-  buildNumber = build;
+  PProcessInstance = this;
 
 #ifdef P_RTEMS
 
@@ -1502,26 +1539,12 @@ PProcess::PProcess(const char * manuf, const char * name,
 
 #else // P_RTEMS
 
-  if (p_argv != 0 && p_argc > 0) {
-    executableFile = p_argv[0];
-    arguments.SetArgs(p_argc-1, p_argv+1);
-  }
-
 #ifdef _WIN32
-  if (executableFile.IsEmpty()) {
-    // Try to get the real image path for this process
-    PVarString moduleName;
-    if (GetModuleFileName(GetModuleHandle(NULL), moduleName.GetPointer(1024), 1024) > 0) {
-      executableFile = moduleName;
-      executableFile.Replace("\\??\\","");
-    }
-  }
-  else {
-    if (!PFile::Exists(executableFile)) {
-      PString execFile = executableFile + ".exe";
-      if (PFile::Exists(execFile))
-        executableFile = execFile;
-    }
+  // Try to get the real image path for this process
+  PVarString moduleName;
+  if (GetModuleFileName(GetModuleHandle(NULL), moduleName.GetPointer(1024), 1024) > 0) {
+    executableFile = moduleName;
+    executableFile.Replace("\\??\\","");
   }
 
 #if defined(P_VIDEO)  && defined(P_VFW_CAPTURE)
@@ -1535,8 +1558,6 @@ PProcess::PProcess(const char * manuf, const char * name,
 
 #endif // P_RTEMS
 
-  InitialiseProcessThread();
-
   Construct();
 
 #if defined(P_MINGW)
@@ -1615,6 +1636,8 @@ void PProcess::PostShutdown()
     delete startup->second;
 
   startups.clear();
+
+  PProcessInstance = NULL;
 }
 
 
@@ -1641,6 +1664,12 @@ void PProcess::OnThreadEnded(PThread & /*thread*/)
 }
 
 
+bool PProcess::OnInterrupt(bool)
+{
+  return false;
+}
+
+
 PBoolean PProcess::IsInitialised()
 {
   return PProcessInstance != NULL;
@@ -1862,6 +1891,24 @@ bool PProcess::HostSystemURLHandlerInfo::Register()
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
 
+PThread * PThread::Current()
+{
+  if (!PProcess::IsInitialised())
+    return NULL;
+
+  PProcess & process = PProcess::Current();
+
+  process.activeThreadMutex.Wait();
+  PThread * thread = process.activeThreads.GetAt((unsigned)GetCurrentThreadId());
+  process.activeThreadMutex.Signal();
+
+  if (thread == NULL)
+    thread = new PExternalThread;
+
+  return thread;
+}
+
+
 void PThread::PrintOn(ostream & strm) const
 {
   strm << GetThreadName();
@@ -2369,5 +2416,24 @@ PWriteWaitAndSignal::~PWriteWaitAndSignal()
   mutex.EndWrite();
 }
 
+/////////////////////////////////////////////////////////////////////////////
+
+PSystemLog::PSystemLog(Level level)   ///< only messages at this level or higher will be logged
+  : iostream(cout.rdbuf()) 
+{ 
+  logLevel = level; 
+  buffer.log = this; 
+  init(&buffer); 
+}
+
+PSystemLog::PSystemLog(const PSystemLog & other)
+  : PObject(other), iostream(cout.rdbuf()) 
+{
+}
+
+PSystemLog & PSystemLog::operator=(const PSystemLog &)
+{ 
+  return *this; 
+}
 
 // End Of File ///////////////////////////////////////////////////////////////
