@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -605,7 +605,7 @@ int PServiceProcess::InitialiseService()
   return -1;
 }
 
-int PServiceProcess::_main(void *)
+int PServiceProcess::InternalMain(void *)
 {
   if ((terminationValue = InitialiseService()) < 0) {
     // Make sure housekeeping thread is going so signals are handled.
