@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Revision: 21766 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-10 12:53:48 +0000 (Wed, 10 Dec 2008) $
  */
 
 #include <ptlib/socket.h>
@@ -90,7 +90,9 @@ static PBoolean PAssertThreadOp(int retval,
     // Give up and assert
   }
 
+#if P_USE_ASSERTS
   PAssertFunc(file, line, NULL, psprintf("Function %s failed", funcname));
+#endif
   return PFalse;
 }
 
@@ -121,9 +123,31 @@ void PHouseKeepingThread::Main()
 
   while (!closing) {
     PTimeInterval delay = process.timers.Process();
+    if (delay > 10000)
+      delay = 10000;
 
     process.breakBlock.Wait(delay);
 
+    process.activeThreadMutex.Wait();
+    PBoolean found;
+    do {
+      found = PFalse;
+      for (PINDEX i = 0; i < process.activeThreads.GetSize(); ++i) {
+        PThread & thread = process.activeThreads.GetDataAt(i);
+        if (thread.autoDelete && thread.IsTerminated()) {
+          // unlock the activeThreadMutex to avoid deadlocks:
+          // if somewhere in the destructor a call to PTRACE() is made,
+          // which itself calls PThread::Current(), deadlocks are possible
+          process.activeThreadMutex.Signal();
+          delete process.activeThreads.RemoveAt(process.activeThreads.GetKeyAt(i));
+          process.activeThreadMutex.Wait();
+          found = PTrue;
+          break;
+        }
+      }
+    } while (found == PTrue); 
+    process.activeThreadMutex.Signal();
+
     process.PXCheckSignals();
   }
 }
@@ -161,8 +185,8 @@ void PProcess::Construct()
   housekeepingThread = NULL;
 
 #ifdef P_MACOSX
-    // records the main thread for priority adjusting
-    baseThread = pthread_self();
+  // records the main thread for priority adjusting
+  baseThread = pthread_self();
 #endif
 
   CommonConstruct();
@@ -214,7 +238,7 @@ PProcess::~PProcess()
 
 PBoolean PProcess::PThreadKill(pthread_t id, unsigned sig)
 {
-  PWaitAndSignal m(threadMutex);
+  PWaitAndSignal m(activeThreadMutex);
 
   if (!activeThreads.Contains((unsigned)id)) 
     return PFalse;
@@ -222,16 +246,20 @@ PBoolean PProcess::PThreadKill(pthread_t id, unsigned sig)
   return pthread_kill(id, sig) == 0;
 }
 
+void PProcess::PXSetThread(pthread_t id, PThread * thread)
+{
+  activeThreadMutex.Wait();
+  PThread * currentThread = activeThreads.GetAt((PINDEX)id);
+  activeThreads.SetAt((PINDEX)id, thread);
+  activeThreadMutex.Signal();
+
+  if (currentThread != NULL) 
+    delete currentThread;
+}
 
 //////////////////////////////////////////////////////////////////////////////
 
 PThread::PThread()
-{
-  // see InitialiseProcessThread()
-}
-
-
-void PThread::InitialiseProcessThread()
 {
   autoDelete          = PFalse;
 
@@ -253,10 +281,18 @@ void PThread::InitialiseProcessThread()
   PAssertOS(::pipe(unblockPipe) == 0);
 #endif
 
-  ((PProcess *)this)->activeThreads.DisallowDeleteObjects();
-  ((PProcess *)this)->activeThreads.SetAt((unsigned)PX_threadId, this);
-
   PX_firstTimeStart = PFalse;
+
+  if (!PProcess::IsInitialised())
+    return;
+
+  autoDelete = true;
+
+  PProcess & process = PProcess::Current();
+
+  process.PXSetThread(PX_threadId, this);
+
+  process.SignalTimerChange();
 }
 
 
@@ -297,9 +333,15 @@ PThread::PThread(PINDEX stackSize,
 
 PThread::~PThread()
 {
+  pthread_t id = PX_threadId;
+
   if (PX_threadId != 0 && PX_threadId != pthread_self())
     Terminate();
 
+#if PTRACING
+  PTrace::Cleanup();
+#endif
+
   PAssertPTHREAD(::close, (unblockPipe[0]));
   PAssertPTHREAD(::close, (unblockPipe[1]));
 
@@ -312,10 +354,15 @@ PThread::~PThread()
   pthread_mutex_unlock(&PX_suspendMutex);
   pthread_mutex_destroy(&PX_suspendMutex);
 
-  if (this != &PProcess::Current())
-    PTRACE(1, "PWLib\tDestroyed thread " << this << ' ' << threadName << "(id = " << ::hex << PX_threadId << ::dec << ")");
-  else
-    PProcessInstance = NULL;
+  // PProcess is a subclass of PThread, if the PProcess instance is deleted,
+  // this destructor runs but PProcessInstance is already set to NULL
+  if (PProcessInstance != NULL) {
+    if (this == PProcessInstance)
+      PProcessInstance = NULL;
+    else {
+      PTRACE_IF(1, !autoDelete, "PWLib\tDestroyed thread " << this << ' ' << threadName << "(id = " << ::hex << id << ::dec << ")");
+    }
+  }
 }
 
 
@@ -360,18 +407,18 @@ void PThread::Restart()
   static PINDEX highWaterMark = 0;
 
   // lock the thread list
-  process.threadMutex.Wait();
+  process.activeThreadMutex.Wait();
 
   // create the thread
   PAssertPTHREAD(pthread_create, (&PX_threadId, &threadAttr, PX_ThreadStart, this));
 
   // put the thread into the thread list
-  process.activeThreads.SetAt((unsigned)PX_threadId, this);
+  process.PXSetThread(PX_threadId, this);
   if (process.activeThreads.GetSize() > highWaterMark)
     newHighWaterMark = highWaterMark = process.activeThreads.GetSize();
 
   // unlock the thread list
-  process.threadMutex.Signal();
+  process.activeThreadMutex.Signal();
 
   PTRACE_IF(4, newHighWaterMark > 0, "PWLib\tThread high water mark set: " << newHighWaterMark);
 
@@ -685,16 +732,6 @@ void PThread::Yield()
 }
 
 
-PThread * PThread::Current()
-{
-  PProcess & process = PProcess::Current();
-  process.threadMutex.Wait();
-  PThread * thread = process.activeThreads.GetAt((unsigned)pthread_self());
-  process.threadMutex.Signal();
-  return thread;
-}
-
-
 void PThread::Terminate()
 {
   if (PX_origStackSize <= 0)
@@ -818,38 +855,38 @@ void PThread::PX_ThreadEnd(void * arg)
   PThread * thread = (PThread *)arg;
   PProcess & process = PProcess::Current();
   process.OnThreadEnded(*thread);
-  process.threadMutex.Wait();
+  
+  // Calls to PTRACE() MUST be made with activeThreadMutex unlocked, or deadlocks may occur.
+  // (PTRACE() itself calls PThread::Current()). Also, PTRACE must be done BEFORE removing the
+  // thread from the activeThreads dictionary, or PTrace::Current() will create a new 
+  // PExternalThread instance, causing the very same deadlock
+  PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << thread->GetThreadName());
+  
+  process.activeThreadMutex.Wait();
 
   pthread_t id = thread->GetThreadId();
   if (id == 0) {
     // Don't know why, but pthreads under Linux at least can call this function
     // multiple times! Probably a bug, but we have to allow for it.
-    process.threadMutex.Signal();
+    process.activeThreadMutex.Signal();
     PTRACE(2, "PWLib\tAttempted to multiply end thread " << thread << " ThreadID=" << (void *)id);
     return;
-  }  
+  }
 
- // remove this thread from the active thread list
+  // remove this thread from the active thread list
   process.activeThreads.SetAt((unsigned)id, NULL);
 
-  // delete the thread if required, note this is done this way to avoid
-  // a race condition, the thread ID cannot be zeroed before the if!
-  PString threadName = thread->GetThreadName();
-  if (thread->autoDelete) {
-    thread->PX_threadId = 0;  // Prevent terminating terminated thread
-    process.threadMutex.Signal();
-    PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << threadName);
+  bool deleteThread = thread->autoDelete; // Get flag before releasing lock
 
+  thread->PX_threadId = 0;  // Prevent terminating terminated thread
+  process.activeThreadMutex.Signal();
+
+  if (deleteThread) {
     /* It is now safe to delete this thread. Note that this thread
-       is deleted after the process.threadMutex.Signal(), which means
-       PWaitAndSignal(process.threadMutex) could not be used */
+       is deleted after the process.activeThreadMutex.Signal(), which means
+       PWaitAndSignal(process.activeThreadMutex) could not be used */
     delete thread;
   }
-  else {
-    thread->PX_threadId = 0;
-    process.threadMutex.Signal();
-    PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << threadName);
-  }
 }
 
 int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
@@ -1202,13 +1239,11 @@ PTimedMutex::PTimedMutex()
   pthread_mutexattr_t attr;
   PAssertPTHREAD(pthread_mutexattr_init, (&attr));
 
-  PAssertPTHREAD(pthread_mutexattr_settype, (&attr,
-#if P_HAS_RECURSIVE_MUTEX == 2
-PTHREAD_MUTEX_RECURSIVE
+#if (P_HAS_RECURSIVE_MUTEX == 2)
+  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE));
 #else
-PTHREAD_MUTEX_RECURSIVE_NP
+  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE_NP));
 #endif
-));
 
   PAssertPTHREAD(pthread_mutex_init, (&mutex, &attr));
   PAssertPTHREAD(pthread_mutexattr_destroy, (&attr));
@@ -1223,13 +1258,11 @@ PTimedMutex::PTimedMutex(const PTimedMutex & /*mut*/)
 #if P_HAS_RECURSIVE_MUTEX
   pthread_mutexattr_t attr;
   PAssertPTHREAD(pthread_mutexattr_init, (&attr));
-  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, 
 #if P_HAS_RECURSIVE_MUTEX == 2
-PTHREAD_MUTEX_RECURSIVE
+  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE));
 #else
-PTHREAD_MUTEX_RECURSIVE_NP
+  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE_NP));
 #endif
-));
 
   PAssertPTHREAD(pthread_mutex_init, (&mutex, &attr));
   PAssertPTHREAD(pthread_mutexattr_destroy, (&attr));
@@ -1328,7 +1361,7 @@ PBoolean PTimedMutex::Wait(const PTimeInterval & waitTime)
 
 #else // P_PTHREADS_XPG6
 
-  do {
+  for (;;) {
     if (pthread_mutex_trylock(&mutex) == 0) {
 #if P_HAS_RECURSIVE_MUTEX == 0
       PAssert((lockerId == (pthread_t)-1) && (lockCount.IsZero()),
@@ -1339,11 +1372,11 @@ PBoolean PTimedMutex::Wait(const PTimeInterval & waitTime)
       return PTrue;
     }
 
+    if (PTime() >= finishTime)
+      return PFalse;
+
     PThread::Current()->Sleep(10); // sleep for 10ms
-  } while (PTime() < finishTime);
-
-  return PFalse;
-
+  }
 #endif // P_PTHREADS_XPG6
 }
 
